require './common.rb'
require './module.rb'
require './server.rb'

require 'yaml'

module Vrobot4
   def self.loadServer servinfo
      type = servinfo["type"]

      serv = Server::get_server_type(type).new(servinfo)

      servinfo["modules"].each \
         {|mod| serv.load_mod mod} if servinfo.key? "modules"

      serv
   end

   def self.loadBot botinfo
      servers = []
      botinfo["servers"].each {|servinfo| servers << loadServer(servinfo)}
      {info: botinfo, servs: servers}
   end

   def self.runBots bots
      thrds = []
      bots.each do |bot|
         bot[:servs].each {|serv| thrds << Thread.new {serv.connect}}
      end
      thrds.each {|th| th.join}
   end

   def self.main cfgname
      cfg = nil

      log :INFO, "vrobot version", Version

      begin
         cfg = YAML.load IO.read(cfgname)
      rescue
         log :ERROR, "error reading bot config:", $!
         return
      end

      Vrobot4.debug = cfg["debug"] if cfg.key? "debug"
      cfg["loadmods"].each {|mod| load mod, true}

      begin
         bots = []
         cfg["bots"].each {|botinfo| bots << loadBot(botinfo)}
         log :DEBUGV, "bots:", bots.to_s
      rescue
         log :ERROR, "error loading bot config:",
            $!.to_s + "\n" + $!.backtrace.first(3).join("\n")
         return
      end

      runBots bots
   end
end

Vrobot4.main "bot.yml"

## EOF
